package Shared.ProgramStateController;

import URServers.RealtimeInterface;
import Shared.Backend.BackendHandler;

import java.util.Timer;
import java.util.TimerTask;

public class StateMonitor {
    private static Timer getStateTimer;
    private static TimerTask collectStateTask;
    
    private boolean pauseFlag = false;
    private RealtimeInterface realtimeInterface;
    private StateReturn stateReturn = new StateReturn();
    private ActOnState oldState = new ActionOnStop();
    private ActOnState runState = new ActionOnStop();
    private BackendHandler backendHandler;

    public StateMonitor(BackendHandler backendHandler) {
        this.backendHandler = backendHandler;
        reset();
    }

    // Resets the state of the StateMonitor
    public void reset() {
        this.realtimeInterface = new RealtimeInterface();
        this.oldState = new ActionOnStop();
        this.runState = new ActionOnStop();
    }

    public void programStateThread() {
        System.out.println("State monitor Started");

        if (getStateTimer != null) {
            collectStateTask.cancel();
            getStateTimer.cancel();
            getStateTimer.purge();
        }

        getStateTimer = new Timer(true);
        collectStateTask = new TimerTask() {
            @Override
            public void run() {
                try {
                    realtimeInterface.readNow();
                    runState = stateReturn.returnState(realtimeInterface.getPrgState()[0]);
                    if (oldState != runState) {
                        actionOnButton(runState);
                    }
                    oldState = runState;
                } catch (Exception e) {
                    // Handle the exception here, e.g., log the error
                    e.printStackTrace();
                    // Stop the monitor thread and attempt to recover
                    stopMonitorThread();
                    reset();  // Reset the state
                    startMonitorThread();
                }
            }
        };
        getStateTimer.scheduleAtFixedRate(collectStateTask, 0, 5000);
    }

    public void startMonitorThread() {
        programStateThread();
    }

    public static void stopMonitorThread() {
        System.out.println("Monitor stopped");
        if (collectStateTask != null) {
            collectStateTask.cancel();
        }
        if (getStateTimer != null) {
            getStateTimer.cancel();
            getStateTimer.purge();
        }
    }

    private void actionOnButton(ActOnState actOnState) {
        actOnState.actOnState(backendHandler);
    }
}
